﻿using System;
using System.Reflection;
using System.Windows.Forms;
using GrapeCity.Competition.Maze;

namespace GrapeCity.Competition.MazeDebuger
{
    [Name("AI Script Editor")]
    [Author("MazeDebuger")]
    public class ScriptEditorAI : IMazeAI
    {
        private IMazeAI userAI;
        private IExplorer currentExplorer;

        public void Initialize()
        {
        }

        public void Terminate()
        {
            if(userAI != null)
            {
                userAI.Terminate();
            }
        }

        public void Think(IMaze maze, IExplorer explorer)
        {
            currentExplorer = explorer;

            ScriptEditorDialog mcd = new ScriptEditorDialog();
            if(mcd.ShowDialog() == DialogResult.OK)
            {
                Assembly dynamicAI = mcd.CompiledAssembly;

                if (dynamicAI == null)
                {
                    explorer.Say("The assembly is null.", null);
                    return;
                }

                Type type = FindFirstAI(dynamicAI);
                if (type == null)
                {
                    explorer.Say("Can not find IMazeAI.", null);
                    return;
                }

                try
                {
                    userAI = Activator.CreateInstance(type) as IMazeAI;
                    if (userAI == null)
                    {
                        explorer.Say("Can not create IMazeAI instance.", null);
                        return;
                    }

                    userAI.Initialize();
                }
                catch (Exception e)
                {
                    explorer.Say("Error on intilize IMazeAI instance. - " + e.Message, null);
                }

                try
                {
                    userAI.Think(maze, explorer);
                }
                catch (Exception e)
                {
                    explorer.Say("Error when think - " + e.Message, e);
                }
            }
        }

        public void Hear(string name, string message, object information)
        {
            try
            {
                if(userAI != null)
                {
                    userAI.Hear(name, message, information);
                }
            }
            catch (Exception e)
            {
                if(currentExplorer!= null)
                {
                    currentExplorer.Say("Error when Hear - " + e.Message, e);
                }
            }
        }

        private static Type FindFirstAI(Assembly assembly)
        {
            try
            {
                foreach (Type type in assembly.GetTypes())
                {
                    MemberInfo[] list = type.GetMembers();
                    Type[] interfaces = type.GetInterfaces();

                    if (type.GetInterface("IMazeAI") == typeof(IMazeAI))
                        return type;
                }
            }
            catch
            {
                return null;
            }

            return null;
        }
    }
}
